
#ifndef BLAHTECH_SUP_DEM_LEVEL_MQH
#define BLAHTECH_SUP_DEM_LEVEL_MQH

#property description "Blahtech Supply Demand Level"
#property link        "http://www.blahtech.co.uk"
#property strict

#include "BlahtechUtils.mqh"

class SupDemLevelIdx
{
public:
   int                TfNum;
   ENUM_TIMEFRAMES    Timeframe;
   SdBufferIndexEnum  SupBarNum;
   SdBufferIndexEnum  SupLow;
   SdBufferIndexEnum  DemBarNum;
   SdBufferIndexEnum  DemHigh;
public:
   SupDemLevelIdx(int tfNum, ENUM_TIMEFRAMES timeframe, SdBufferIndexEnum supBarNum, SdBufferIndexEnum supLowdx, SdBufferIndexEnum demBarNum, SdBufferIndexEnum demHigh) :
      TfNum(tfNum),
      Timeframe(timeframe),
      SupBarNum(supBarNum),
      SupLow(supLowdx),
      DemBarNum(demBarNum),
      DemHigh(demHigh)
   {
   }
   
   bool IsValid() const
   {
      return Timeframe != PERIOD_CURRENT;
   }
   
   bool PopulateScoresAndRetouches() const
   {
      return TfNum < 2;
   }
};

class SupDemLevel
{
public:
   int      BarNum;
   int      Score;
   int      Retouch;
   bool     IsStrong;
   datetime Timestamp;
   double   PriceHigh;
   double   PriceLow;

public:
   void     IncrementBarNums()         { BarNum++; }
   string   KeysStr    (bool fixedOnly) const;
   bool     CompareKeys(const SupDemLevel& rhs, bool fixedOnly) const;
   
   static bool CompareLevels(const SupDemLevel* &leftArray[], const SupDemLevel* &rightArray[], bool fixedOnly, int logLevel);
};


bool
SupDemLevel::CompareKeys(const SupDemLevel& rhs, bool fixedOnly) const
{
   return BarNum == rhs.BarNum && Timestamp == rhs.Timestamp && PriceHigh == rhs.PriceHigh && PriceLow == rhs.PriceLow &&
         (fixedOnly || (Score == rhs.Score && Retouch == rhs.Retouch && IsStrong == rhs.IsStrong));
}

string
SupDemLevel::KeysStr(bool fixedOnly) const
{
   string ret = IntegerToString(BarNum) + " " + TimeToString(Timestamp) + " " + DoubleToString(PriceHigh, _Digits) + " " + DoubleToString(PriceLow, _Digits);
   return fixedOnly ? ret : ret + " " + IntegerToString(Score) + " " + IntegerToString(Retouch) + " " + IntegerToString(IsStrong);
}


bool
SupDemLevel::CompareLevels(const SupDemLevel* &leftArray[], const SupDemLevel* &rightArray[], bool fixedOnly, int logLevel)
{
   int  size = ArraySize(leftArray);
   bool ret = size == ArraySize(rightArray);
   
   if (ret)
   {
      for(int i = 0; i < size; i++)
      {
         if (!leftArray[i].CompareKeys(rightArray[i], fixedOnly))
         {
            if (logLevel >= LogError)
            {
               Print("ERROR CompareLevels lhs[", i, "] ", leftArray[i].KeysStr(fixedOnly));
               Print("ERROR CompareLevels rhs[", i, "] ", rightArray[i].KeysStr(fixedOnly));
            }
            ret = false;
            break;
         }
      }
   }
   else
   {
      int compareSize = MathMin(size, ArraySize(rightArray));
      
      for(int i = 0; i < compareSize; i++)
      {
         if (!leftArray[i].CompareKeys(rightArray[i], fixedOnly))
         {
            if (logLevel >= LogError)
            {
               Print("ERROR CompareLevels lhs[", i, "] ", leftArray[i].KeysStr(fixedOnly));
               Print("ERROR CompareLevels rhs[", i, "] ", rightArray[i].KeysStr(fixedOnly));
            }
            break;
         }
         else if (logLevel >= LogError)
         {
            Print("INFO  CompareLevels lhs[", i, "] ", leftArray[i].KeysStr(fixedOnly));
            Print("INFO  CompareLevels rhs[", i, "] ", rightArray[i].KeysStr(fixedOnly));
         }
      }
      if (logLevel >= LogError) { Print("ERROR CompareLevels ArraySize(leftArray)=", size, "  !=  ArraySize(rightArray)=", ArraySize(rightArray)); }
   }
   return ret;
}
